#!/usr/bin/perl -w
use strict;
use warnings;

#my $infile = "otherInfiles/infileNUM.txt";
my $infile = "infile.txt";

####
my $motif = "T..CACCT";
#my $motif = "T[GA][GA]CACCT";
my $minimumHitsNum = 1;
####
#my $targetRegion           = "All";
my $targetRegion           = "45811278..45811338";
my $seedSpeciesName        = "Mouse";
my $transStartSite         = 16;
my $transStartSite_genomic = 45811288;

my $outfile = "outfile.html";

my $oneLineLength = 60;
my $numEverySite  = 10;
my $maxNameLength = 15;
####


open(INFILE,"$infile") or die "$!";
my @inFileCont = <INFILE>;
close INFILE;

my ($ref_names, $ref_seqs) = &fasReader2D(\@inFileCont);
my $ref_numTable           = &numCorresponder($seedSpeciesName, $transStartSite, $transStartSite_genomic, $ref_names, $ref_seqs);
($ref_names, $ref_seqs)    = &indexing($ref_numTable, $numEverySite, $ref_names, $ref_seqs);

############# ClippingRegion
my ($targetStartNum, $targetEndNum) = &targetIdentification($targetRegion, $ref_numTable);


############# Motif finding
my $match_revCom = &revCom($motif);
my $refPosRecs= &motifFinder($motif, $match_revCom, $seedSpeciesName, $targetStartNum, $targetEndNum, $ref_names, $ref_seqs);

############# Sequence printout
&print_positions($seedSpeciesName, $targetStartNum, $targetEndNum, $ref_numTable, $ref_names, $refPosRecs);

#$ref_seqs  = &matchFirst($seedSpeciesName, $ref_names, $ref_RecClipped);
$ref_names = &white2names($maxNameLength, $ref_names);
printingHTML($outfile, $ref_numTable, $oneLineLength, $targetStartNum, $targetEndNum, $ref_names, $ref_seqs, $refPosRecs);

exit;
#############################


sub targetIdentification {
  my ($targetRegion, $ref_numTableSR) = @_;
  my @numTableSR = @$ref_numTableSR;
  my $totalLength = scalar(@numTableSR)-1;

  my $startNumSR = 0; my $endNumSR = 0;
  if($targetRegion eq "All"){
    print "Search for all region.\n";
    $startNumSR = 0; $endNumSR = $totalLength;
  } else {
    my($start_gPos, $end_gPos) = split(/\.\./,$targetRegion);
    my $start_gPosTemp = &tSeparator($start_gPos);
    my $end_gPosTemp   = &tSeparator($end_gPos);
    for(my $num=0; $num<scalar(@numTableSR);$num++){
      if ($start_gPos == $numTableSR[$num][2]){
        $startNumSR = $num; last;
      }
    }
    for(my $num=scalar(@numTableSR)-1; $num>=0; $num--){
      if ($end_gPos == $numTableSR[$num][2]){
        $endNumSR = $num; last;
      }
    }
  }
  
  return($startNumSR, $endNumSR);
}


sub motifFinder {
  my($motif1SR, $motif2SR, $seedSpeciesNameSR, $targetStartNumSR, $targetEndNumSR, $ref_names, $ref_seqs) = @_;
  my @namesSR = @$ref_names;
  my @seqsSR  = @$ref_seqs;
  my $speciesAmount = scalar(@namesSR);
  my $totalLength   = $targetEndNumSR-$targetStartNumSR;

  my @tempMotif1 = split(//, $motif1SR);
  $motif1SR = join('-{0,10}', @tempMotif1);
  my @tempMotif2 = split(//, $motif2SR);
  $motif2SR = join('-{0,10}',@tempMotif2);
  
  my @matchPositions = ();
  for(my $i=0; $i < $speciesAmount; $i++){
    if ($namesSR[$i] eq ">GPOSIT"){
      push(@matchPositions, "NONE"),
    } 
    else{
      my $str = join('', @{$seqsSR[$i]});
      $str = substr($str, $targetStartNumSR, $totalLength);
      my $str1 = uc $str;
      my %tempMotifRec = ();
      if($str1 =~ /$motif1SR/ || $str1 =~ /$motif2SR/){
        my $str2 = $str1;
        while($str1 =~ /$motif1SR/){
           my $formerSeqLength = length($`);
           my $matchSeqLength = length($&);
           $tempMotifRec{$formerSeqLength+$targetStartNumSR} = $&;
           my $corting = "X" x $matchSeqLength;
           $str1 =~ s/$motif1SR/$corting/;
        }
        while($str2 =~ /$motif2SR/){
           my $formerSeqLength = length($`);
           my $matchSeqLength = length($&);
           $tempMotifRec{$formerSeqLength+$targetStartNumSR} = $&;
           my $corting = "X" x $matchSeqLength;
           $str2 =~ s/$motif2SR/$corting/;
        }
        push(@matchPositions, \%tempMotifRec);
      }
      else{
        push(@matchPositions, "NONE");
      }
    }
  }
  return (\@matchPositions);
}


sub reverseR {
  my ($seqLine) = @_;
  my $revcomp = reverse($seqLine);
  $revcomp =~ s/\[/1/g;
  $revcomp =~ s/\]/2/g;
  $revcomp =~ s/1/\]/g;
  $revcomp =~ s/2/\[/g;
  return ($revcomp);
}


sub revCom {
  my ($seqLine) = @_;
  my $revcomp = reverseR($seqLine);
  $revcomp =~ tr/ATGCatgc
                /TACGtacg/;
  $revcomp =~ tr/atgc/ATGC/;
  return ($revcomp);
}


sub print_positions{
  my($seedSpeciesNameSR, $targetStartNumSR, $targetEndNumSR, $ref_numTableSR, $ref_namesSR, $refPosRecsSR) = @_;
  my @numTableSR = @$ref_numTableSR;
  my @posRecsSR   = @$refPosRecsSR;

  my $referenceNum = &seedRecPicker($seedSpeciesNameSR, $ref_namesSR);

#for(my $i=0; $i<scalar(@posRecsSR); $i++){
#  print "$i\n";
#  if($posRecsSR[$i] eq "NONE"){
#    print "  NONE\n";
#  }
#  else{
#    my %tempH = %{$posRecsSR[$i]};
#    foreach my $ele (keys %tempH) {
#      print "  $ele $tempH{$ele}\n";
#    }
#  }
#}
#exit;
  
  my @matchsSeedSR = (); my @matchsSR = ();
  for(my $i=0; $i<scalar(@posRecsSR); $i++){
    if ($i eq $referenceNum) {
      next if $posRecsSR[$i] eq "NONE";
      my %tempH = %{$posRecsSR[$i]};
      push(@matchsSeedSR, keys %tempH);
    }
    else {
      next if $posRecsSR[$i] eq "NONE";
      my %tempH = %{$posRecsSR[$i]};
      push(@matchsSR, keys %tempH);
    }  
  }
  
  for(my $num=$targetStartNumSR; $num<=$targetEndNumSR; $num++){
    my $hits = grep($_ eq $num, @matchsSR);
    if(grep($_ eq $num, @matchsSeedSR)){
      my $gPositionSeedTemp = &tSeparator($numTableSR[$num][2]);
      print "  $seedSpeciesNameSR|1|\tOthers|$hits|\tFromTSS|".$numTableSR[$num][1]."|\tGenomicPosition|$gPositionSeedTemp|\n";
    }
    else{
      if($hits >= $minimumHitsNum){
        my $gPositionTemp = &tSeparator($numTableSR[$num][2]);
        print "  $seedSpeciesNameSR|0|\tOthers|$hits|\tFromTSS|".$numTableSR[$num][1]."|\tGenomicPosition|$gPositionTemp|\n";
      }
    }
  }
}



sub printingHTML {
  my ($outfileSR, $refNumTableSR, $oneLineLengthSR, $targetStartNumSR, $targetEndNumSR, $ref_namesSR, $ref_seqsSR, $refPosRecsSR) = @_;
  my @namesSR      = @$ref_namesSR;
  my @seqsSR = @$ref_seqsSR;
  my @posRecsSR    = @$refPosRecsSR;
  my @numTableSR   = @$refNumTableSR;
  my $speciesAmount = scalar(@namesSR);
  my $totalLengthSR = $targetEndNumSR - $targetStartNumSR + 1;

  my ($head, $tail) = &variables;
  
  my $outfileTXT = $outfileSR.".txt";


#for(my $i=0; $i<scalar(@posRecsSR); $i++){
#  print "$i\n";
#  if($posRecsSR[$i] eq "NONE"){
#    print "  NONE\n";
#  }
#  else{
#    my %tempH = %{$posRecsSR[$i]};
#    foreach my $ele (keys %tempH) {
#      print "  $ele $tempH{$ele}\n";
#    }
#  }
#}
#exit;

  for(my $i=0; $i<scalar(@posRecsSR); $i++){
    next if $posRecsSR[$i] eq "NONE";
    my %tempH = %{$posRecsSR[$i]};
    foreach my $ele (keys %tempH) {
      #print "  $ele $tempH{$ele}\n";
      my $lengthTEMP = length($tempH{$ele});
      #print "  len $lengthTEMP\n";
      #print "  $seqClippedSR[$i][$ele-$targetStartNumSR]\n";
      for(my $k=0; $k<$lengthTEMP; $k++){
        $seqsSR[$i][$ele+$k] = "<span class=mot>".$seqsSR[$i][$ele+$k]."</span>";
      }
    }
  }


  #for(my $i=0; $i<scalar(@seqsSR); $i++){
  #  print "$i";
  #  for(my $j=0; $j<scalar(@{$seqsSR[1]}); $j++){
  #    print "$seqsSR[$i][$j]";
  #  }
  #  print "\n";
  #}

  my @printLines = ();
  push(@printLines, ($speciesAmount-1)." $totalLengthSR");
  push(@printLines, "$numTableSR[$targetStartNumSR][1]..$numTableSR[$targetEndNumSR][1]");
  my $startGpositionTemp = &tSeparator($numTableSR[$targetStartNumSR][2]);
  my $endGpositionTemp = &tSeparator($numTableSR[$targetEndNumSR][2]);
  push(@printLines,"$startGpositionTemp..$endGpositionTemp");

  my $lineLastSite = 0;
  #print "targetStartNumSR $targetStartNumSR\n";
  #print "targetEndNumSR $targetEndNumSR\n";
  for (my $num=$targetStartNumSR; $num<$targetEndNumSR;) {
    if(($num + $oneLineLengthSR) > $targetEndNumSR){
      $lineLastSite = $targetEndNumSR+1;
    }
    else{
      $lineLastSite = $num + $oneLineLengthSR;
    }
    #print "lineLastSite $lineLastSite\n";
  
    for (my $sp=0; $sp<$speciesAmount; $sp++) {

      my $sequenceTEM = "";
      my $spaciesName = $namesSR[$sp];
      if($spaciesName =~ /^GPOSIT/){
        my $gPositionTemp = &tSeparator($numTableSR[$num][2]);
        my $numChar = length($gPositionTemp);
        $spaciesName =~ s/GPOSIT/      /;
        $spaciesName =~ s/ {$numChar}/$gPositionTemp/;
      }
      $sequenceTEM .= $spaciesName;
 
      #print "$sp\n";
      for (my $num_line=$num; $num_line < $lineLastSite; $num_line++){
        #print "  num_line $num_line\n";
        $sequenceTEM .= $seqsSR[$sp][$num_line];
      }
      push(@printLines, $sequenceTEM);
      $sequenceTEM = "";
      #print "\n";
    }
  
    $num = $lineLastSite;
    push(@printLines, "\n");
  }

  open(OUT, ">$outfileSR");
  print OUT "$head\n";
  foreach my $line (@printLines) {
    chomp($line);
    print OUT "$line\n";
  }
  print OUT "$tail\n";
  close OUT;

  open(OUTT, ">$outfileTXT");
  foreach my $line (@printLines) {
    chomp($line);
    my $lineTMP = $line;
    $lineTMP =~ s/<[^>]*>//g;
    print OUTT "$lineTMP\n";
  }
  close OUTT;
}


sub tSeparator{
  my ($numSR) = @_;
  1 while $numSR =~ s/(.*\d)(\d\d\d)/$1,$2/;
  return($numSR);
}


sub seedRecPicker{
  my($seedSpeciesNameSR, $ref_namesSR) = @_;
  my @namesSR = @$ref_namesSR;
  my $hitNum = "";
  for(my $i = 0; $i<scalar(@namesSR); $i++){
    if($namesSR[$i] eq ">".$seedSpeciesNameSR){
      $hitNum = $i;
      last;
    }
  }
  return ($hitNum);
}


sub numCorresponder {
  my ($seedSpeciesNameSR, $transStartSiteSR, $transStartSite_genomicSR, $ref_namesSR, $ref_seqsSR) = @_;
  my @seqsSR = @$ref_seqsSR;
  my $positionGenomic  = $transStartSite_genomicSR - $transStartSiteSR;
  my $positionRelative = -1 * $transStartSiteSR;

  my $referenceNum = &seedRecPicker($seedSpeciesNameSR, $ref_namesSR);
  my @numTableSR = ();
  my $totalLength = scalar(@{$seqsSR[$referenceNum]});
  for(my $i=0; $i<$totalLength; $i++){
    # print "Char $seqsSR[$referenceNum][$i], i $i, relative $positionRelative, genomic $positionGenomic\n";
    push(@numTableSR, [$seqsSR[$referenceNum][$i], $positionRelative, $positionGenomic]);
    if ($i+1 < $totalLength && $seqsSR[$referenceNum][$i] ne "-"){
      $positionGenomic++; $positionRelative++; 
    }
  }
  return(\@numTableSR);
}


sub num2charArray {
  my ($number) = @_;
  my $sign = "";
  if($number < 0){
    $sign = "-";
    $number = -1 * $number;
  }
  
  my @nums = ();
  if($number == 0) {
    @nums = ($number, @nums);
  } 
  elsif($number == 100){
    @nums = (1,0,0);
  }
  elsif($number == 10){
    @nums = (1,0);
  }
  else {
    while($number > 1){
      my $num = $number % 10;
      @nums = ($num, @nums);
      $number = $number/10;
    }
  }
  if($sign eq "-"){
    @nums = ("-", @nums);
  }
  return (\@nums);
}


sub indexing{

#  my($refNumTableSR, $numEverySiteSR, $refRecSR) = @_;
  my($refNumTableSR, $numEverySiteSR, $ref_namesSR, $ref_seqsSR) = @_;
  my @numTableSR = @$refNumTableSR;
  my $totalLength = scalar(@numTableSR);
  #my @recSR = @$refRecSR;
  my @namesSR = @$ref_namesSR;
  my @seqsSR  = @$ref_seqsSR;

  @namesSR = ('>GPOSIT', @namesSR);
  my @temp = ();
  for(my $j=0; $j<$totalLength; $j++){
    $temp[$j] = "_";
  }
  @seqsSR = (\@temp, @seqsSR);

  my $flag = 0;
  for(my $num=0; $num < $totalLength; ){
    if($numTableSR[$num][1] % $numEverySiteSR == 0){
      my $ref_numCharArray = &num2charArray($numTableSR[$num][1]);
      my @numChars = @$ref_numCharArray;
      if($flag == 0){
        for(my $k=0; $k<scalar(@numChars); $k++){
          #print "  a seqsSR[0][$num] = |$numChars[$k]|\n";
          $seqsSR[0][$num] = $numChars[$k];
          $num++;
          last if $num >= $totalLength;
        }
        $flag = 1;
      }
      else {
        #print "  b seqsSR[0][$num] = |_|\n";
        $seqsSR[0][$num] = "_";
        $num++;
      }
    } 
    else {
      $flag = 0 ;
      #print "  c seqsSR[0][$num] = |_|\n";
      $seqsSR[0][$num] = "_";
      $num++;
    }
  }

  return(\@namesSR, \@seqsSR);
}


sub white2names{
  my($maxNameLengthSR, $ref_namesSR)=@_;
  for my $ele (@$ref_namesSR) {
    $ele =~ s/^[<|>]//;
    my $white = " " x ($maxNameLength - length($ele));
    $ele = $ele.$white;
  }
  return($ref_namesSR);
}


sub matchFirst{
  my($seedSpeciesNameSR, $ref_namesSR, $ref_seqsSR)=@_;
  my @seqsSR = @$ref_seqsSR;
  my $speciesAmount  = scalar(@seqsSR)-1;
  my $totalLength    = scalar(@{$seqsSR[0]})-1;

  my $referenceNum = &seedRecPicker($seedSpeciesNameSR, $ref_namesSR);

  for (my $i=0; $i<$speciesAmount; $i++){
    next if $i == $referenceNum;
    for(my $j=0; $j<$totalLength; $j++){
      next if $seqsSR[$referenceNum][$j] =~ /[-NX]/;
      $seqsSR[$i][$j] = "." if $seqsSR[$i][$j] eq $seqsSR[$referenceNum][$j];
    }
  }
  return(\@seqsSR);
}


sub fasReader2D
{
  my ($inFileContSR) = @_;

  my $name = "";
  my $seq  = "";
  my $flag = 0;

  my @namesSR = ( );
  my @seqsSR  = ( );

  my $i = 0;
  foreach my $line (@$inFileContSR) {
    chomp($line);
    if($line =~ /^[<|>]/){  
      if($flag == 0){
        $name = $line;
        $flag = 1;  
      } else {
        $namesSR[$i] = $name;
        my @seqTemp = split(//,$seq);
        $seqsSR[$i]  = \@seqTemp;
        #print "seqsSR[$i] |@{$seqsSR[$i]}|\n";
        $i++;  

        $name = "$line";
        $seq = "";
      }
    } else {
      $seq .= $line; 
    }
  }
  $namesSR[$i] = $name;  
  my @seqTemp = split(//,$seq);
  $seqsSR[$i] = \@seqTemp;

  return (\@namesSR, \@seqsSR);
}


sub variables {
my $headSR = 
'
<!DOCTYPE html>
<html><head>
    <meta http-equiv="Content-Type" content="text/html">
    <title>motifFinder_region</title>
    <style type="text/css">
    .mot { background-color: #000; color: white}
    </style>
  </head>

  <body>
	<pre>';

  my $tailSR = 
  '
  	  </pre>
    </body>
  </html>
  ';

  return($headSR, $tailSR);
}